(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['vue-lazy-hydration'] = {}));
}(this, (function (exports) { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);

    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      if (enumerableOnly) symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
      keys.push.apply(keys, symbols);
    }

    return keys;
  }

  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};

      if (i % 2) {
        ownKeys(Object(source), true).forEach(function (key) {
          _defineProperty(target, key, source[key]);
        });
      } else if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
      } else {
        ownKeys(Object(source)).forEach(function (key) {
          Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
      }
    }

    return target;
  }

  var observers = new Map();
  function makeHydrationObserver(options) {
    if (typeof IntersectionObserver === "undefined") return null;
    var optionKey = JSON.stringify(options);
    if (observers.has(optionKey)) return observers.get(optionKey);
    var observer = new IntersectionObserver(function (entries) {
      entries.forEach(function (entry) {
        // Use `intersectionRatio` because of Edge 15's
        // lack of support for `isIntersecting`.
        // See: https://github.com/w3c/IntersectionObserver/issues/211
        var isIntersecting = entry.isIntersecting || entry.intersectionRatio > 0;
        if (!isIntersecting || !entry.target.hydrate) return;
        entry.target.hydrate();
      });
    }, options);
    observers.set(optionKey, observer);
    return observer;
  }

  function makeHydrationPromise() {
    var hydrate = function hydrate() {};

    var hydrationPromise = new Promise(function (resolve) {
      hydrate = resolve;
    });
    return {
      hydrate: hydrate,
      hydrationPromise: hydrationPromise
    };
  }

  var isServer = typeof window === "undefined";

  function isAsyncComponentFactory(componentOrFactory) {
    return typeof componentOrFactory === "function";
  }

  function resolveComponent(componentOrFactory) {
    if (isAsyncComponentFactory(componentOrFactory)) {
      return componentOrFactory().then(function (componentModule) {
        return componentModule.default;
      });
    }

    return componentOrFactory;
  }

  function makeNonce(_ref) {
    var component = _ref.component,
        hydrate = _ref.hydrate,
        hydrationPromise = _ref.hydrationPromise;
    return function () {
      return new Promise(function (resolve) {
        if (isServer) hydrate();
        hydrationPromise.then(function () {
          return resolve(resolveComponent(component));
        });
      });
    };
  }

  function makeHydrationBlocker(component, options) {
    return _objectSpread2(_objectSpread2({}, options), {}, {
      mixins: [{
        beforeCreate: function beforeCreate() {
          this.cleanupHandlers = [];

          var _makeHydrationPromise = makeHydrationPromise(),
              hydrate = _makeHydrationPromise.hydrate,
              hydrationPromise = _makeHydrationPromise.hydrationPromise;

          this.Nonce = makeNonce({
            component: component,
            hydrate: hydrate,
            hydrationPromise: hydrationPromise
          });
          this.hydrate = hydrate;
          this.hydrationPromise = hydrationPromise;
        },
        beforeDestroy: function beforeDestroy() {
          this.cleanup();
        },
        mounted: function mounted() {
          var _this = this;

          if (this.$el.nodeType === Node.COMMENT_NODE) {
            // No SSR rendered content, hydrate immediately.
            this.hydrate();
            return;
          }

          if (this.never) return;

          if (this.whenVisible) {
            var observerOptions = this.whenVisible !== true ? this.whenVisible : undefined;
            var observer = makeHydrationObserver(observerOptions);

            if (!observer) {
              this.hydrate();
              return;
            }

            this.$el.hydrate = this.hydrate;

            var cleanup = function cleanup() {
              return observer.unobserve(_this.$el);
            };

            this.hydrationPromise.then(cleanup);
            observer.observe(this.$el);
            return;
          }

          if (this.whenIdle) {
            // If `requestIdleCallback()` or `requestAnimationFrame()`
            // is not supported, hydrate immediately.
            if (!("requestIdleCallback" in window) || !("requestAnimationFrame" in window)) {
              // eslint-disable-next-line no-underscore-dangle
              this.hydrate();
              return;
            } // @ts-ignore


            var id = requestIdleCallback(function () {
              // eslint-disable-next-line no-underscore-dangle
              requestAnimationFrame(_this.hydrate);
            }, {
              timeout: this.idleTimeout
            }); // @ts-ignore

            var _cleanup = function _cleanup() {
              return cancelIdleCallback(id);
            };

            this.hydrationPromise.then(_cleanup);
          }

          if (this.interactionEvents.length) {
            var eventListenerOptions = {
              capture: true,
              once: true,
              passive: true
            };
            this.interactionEvents.forEach(function (eventName) {
              var _this$$el;

              var eventListenerParams = [eventName, _this.hydrate, eventListenerOptions];

              (_this$$el = _this.$el).addEventListener.apply(_this$$el, eventListenerParams);

              _this.cleanupHandlers.push(function () {
                var _this$$el2;

                return (_this$$el2 = _this.$el).removeEventListener.apply(_this$$el2, eventListenerParams);
              });
            });
          }
        },
        methods: {
          cleanup: function cleanup() {
            this.cleanupHandlers.forEach(function (handler) {
              return handler();
            });
          }
        },
        render: function render(h) {
          return h(this.Nonce, {
            props: this.$attrs
          }, this.$slots.default);
        }
      }]
    });
  }

  function hydrateWhenIdle(componentOrFactory) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref$timeout = _ref.timeout,
        timeout = _ref$timeout === void 0 ? 2000 : _ref$timeout;

    return makeHydrationBlocker(componentOrFactory, {
      beforeCreate: function beforeCreate() {
        this.whenIdle = true;
        this.idleTimeout = timeout;
      }
    });
  }
  function hydrateWhenVisible(componentOrFactory) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$observerOptions = _ref2.observerOptions,
        observerOptions = _ref2$observerOptions === void 0 ? undefined : _ref2$observerOptions;

    return makeHydrationBlocker(componentOrFactory, {
      beforeCreate: function beforeCreate() {
        this.whenVisible = observerOptions || true;
      }
    });
  }
  function hydrateNever(componentOrFactory) {
    return makeHydrationBlocker(componentOrFactory, {
      beforeCreate: function beforeCreate() {
        this.never = true;
      }
    });
  }
  function hydrateOnInteraction(componentOrFactory) {
    var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref3$event = _ref3.event,
        event = _ref3$event === void 0 ? "focus" : _ref3$event;

    var events = Array.isArray(event) ? event : [event];
    return makeHydrationBlocker(componentOrFactory, {
      beforeCreate: function beforeCreate() {
        this.interactionEvents = events;
      }
    });
  }
  var Placeholder = {
    render: function render() {
      return this.$slots.default;
    }
  };
  var LazyHydrate = makeHydrationBlocker(Placeholder, {
    props: {
      idleTimeout: {
        default: 2000,
        type: Number
      },
      never: {
        type: Boolean
      },
      onInteraction: {
        type: [Array, Boolean, String]
      },
      triggerHydration: {
        default: false,
        type: Boolean
      },
      whenIdle: {
        type: Boolean
      },
      whenVisible: {
        type: [Boolean, Object]
      }
    },
    computed: {
      interactionEvents: function interactionEvents() {
        if (!this.onInteraction) return [];
        if (this.onInteraction === true) return ["focus"];
        return Array.isArray(this.onInteraction) ? this.onInteraction : [this.onInteraction];
      }
    },
    watch: {
      triggerHydration: {
        immediate: true,
        handler: function handler(isTriggered) {
          if (isTriggered) this.hydrate();
        }
      }
    }
  });

  exports.default = LazyHydrate;
  exports.hydrateNever = hydrateNever;
  exports.hydrateOnInteraction = hydrateOnInteraction;
  exports.hydrateWhenIdle = hydrateWhenIdle;
  exports.hydrateWhenVisible = hydrateWhenVisible;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
